#! /usr/bin/env python
import serial
import json
import threading
import socket
import datetime
import time
import os
import httplib
import xdrlib
import paho.mqtt.client as mqtt_paho
import mysql.connector
import logging
import getopt
import sys
from pymongo import MongoClient
from random import randint
from binascii import hexlify, unhexlify
from xbee.thread import XBee, ZigBee
from coapthon import defines
from coapthon.resources.resource import Resource
from coapthon.server.coap import CoAP
from coapthon.client.helperclient import HelperClient
from coapthon.utils import parse_uri


class Db:

    sql_config = {
        'user': 'XXXXXX',
        'password': 'XXXXXXXX',
        'host': '127.0.0.1',
        'database': 'iot',
        'raise_on_warnings': True,
        'use_pure': False,
        'buffered': True,
    }

    nosql_config = "mongodb://gateway:raspberry@uoc-shard-00-00-qwzw3.mongodb.net:27017,uoc-shard-00-01-qwzw3.mongodb.net:27017,uoc-shard-00-02-qwzw3.mongodb.net:27017/test?ssl=true&replicaSet=UOC-shard-0&authSource=admin&retryWrites=true"

    def __init__(self):
        pass

    @classmethod
    def save_sensor_data(cls, message):
        thread_one = threading.Thread(target=Db.sql_worker, args=(message,))
        thread_one.daemon = True
        thread_one.start()
        thread_two = threading.Thread(target=Db.nosql_worker, args=(message,))
        thread_two.daemon = True
        thread_two.start()

    @classmethod
    def nosql_worker(cls, message):
        try:
            mongodb_conn = MongoClient(cls.nosql_config)
        except:
            return None
        iot = mongodb_conn.iot
        iot_data = iot.data
        data = {"topic": message["TOPIC"],
                "value": float(message["VALUE"]),
                "date": datetime.datetime.utcnow()}
        data_id = iot_data.insert_one(data).inserted_id
        logging.info('GATEWAY -> MONGODB : ' + message['TOPIC'] + ' '  + str(data_id))

    @classmethod
    def sql_worker(cls, message):
        try:
            float(message["VALUE"])
        except:
            return None
        sensor_id = cls.get_sensor_id(message)
        if sensor_id is not None:
            data_id = Db.create_data(sensor_id, message["VALUE"])
            logging.info('GATEWAY -> MARIADB : ' + message['TOPIC'] + ' ' + str(data_id))

    @classmethod
    def get_location_id(cls, message):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "SELECT id FROM location WHERE name = %s LIMIT 1"
        cursor.execute(query, (message['LOCATION'],))
        row = cursor.fetchone()
        if row is None:
            location_id = Db.create_location(message['LOCATION'])
        else:
            location_id = row[0]
        cursor.close()
        cnx.close()
        return location_id

    @classmethod
    def get_node_id(cls, message):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "SELECT node.id FROM node LEFT JOIN location ON node.location_id = location.id WHERE node.name = %s AND location.name = %s LIMIT 1"
        cursor.execute(query, (message['NODE'], message['LOCATION']))
        row = cursor.fetchone()
        if row is None:
            location_id = Db.get_location_id(message)
            node_id = Db.create_node(location_id, message["NODE"], message["PROTOCOL"], message["ADDRESS"])
        else:
            node_id = row[0]
        cursor.close()
        cnx.close()
        return node_id

    @classmethod
    def get_sensor_id(cls, message):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "SELECT sensor_id FROM sensor_full WHERE topic = %s LIMIT 1"
        cursor.execute(query, (message['TOPIC'],))
        row = cursor.fetchone()
        if row is None:
            node_id = Db.get_node_id(message)
            sensor_id = Db.create_sensor(node_id, message["PARAMETER"])
        else:
            sensor_id = row[0]
        cursor.close()
        cnx.close()
        return sensor_id

    @classmethod
    def create_data(cls, sensor_id, sensor_data):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "INSERT INTO data (sensor_id, sensor_data) VALUES (%s, %s)"
        cursor.execute(query, (sensor_id, sensor_data,))
        cnx.commit()
        data_id = cursor.lastrowid
        cursor.close()
        cnx.close()
        logging.debug('++ DATA SAVED IN MARIADB ' + str(data_id))
        return data_id

    @classmethod
    def create_location(cls, location):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "INSERT INTO location (name, description) VALUES (%s, %s)"
        cursor.execute(query, (location, location))
        cnx.commit()
        return cursor.lastrowid

    @classmethod
    def create_node(cls, location_id, node, protocol, address):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "INSERT INTO node (location_id, name, description, protocol, address) VALUES (%s, %s, %s, %s, %s)"
        cursor.execute(query, (location_id, node, node, protocol, address))
        cnx.commit()
        return cursor.lastrowid

    @classmethod
    def create_sensor(cls, node_id, sensor):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        query = "INSERT INTO sensor (node_id, name, description) VALUES (%s, %s, %s)"
        cursor.execute(query, (node_id, sensor, sensor))
        cnx.commit()
        return cursor.lastrowid

    @classmethod
    def get_node_protocol(cls, location, node):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        location_node = location + "/" + node + "/%"
        query = "SELECT node_protocol FROM sensor_full WHERE topic LIKE %s LIMIT 1"
        cursor.execute(query, (location_node,))
        row = cursor.fetchone()
        cursor.close()
        cnx.close()
        if row is None:
            return None
        else:
            return row[0]


    @classmethod
    def get_node_address(cls, location, node):
        cnx = mysql.connector.connect(**cls.sql_config)
        cursor = cnx.cursor()
        lcation_node = location + "/" + node + "/%"
        query = "SELECT node_address FROM sensor_full WHERE topic LIKE %s LIMIT 1"
        cursor.execute(query, (lcation_node,))
        row = cursor.fetchone()
        cursor.close()
        cnx.close()
        if row is None:
            return None
        else:
            return row[0]


class Rest:

    def __init__(self):
        logging.info('OPENING REST SOCKET')
        self.socket_lock = threading.Lock()
        thread = threading.Thread(target=self.socket,)
        thread.daemon = True
        thread.start()

    def socket(self):
        count = 0
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        sock_file = "/var/www/html/tmp/iotgateway.sock"
        try:
            os.remove(sock_file)
        except OSError:
            logging.warning('Failed deleting old socket')
            pass
        sock.bind(sock_file)
        os.chmod(sock_file, 0777)
        logging.info('SOCKET OPEN')
        sock.listen(5)
        while True:
            (client, address) = sock.accept()
            count +=1
            self.socket_lock.acquire()
            conn_thread = threading.Thread(target=self.socket_worker, args=(client, count))
            conn_thread.daemon = True
            conn_thread.start()
        client.close()


    def socket_worker(self, c, b):
        while True:
            data = c.recv(256)
            if not data:
                self.socket_lock.release()
                break
            try:
                msg = json.loads(data)
            except:
                msg = None
            if msg is not None:
                if msg.has_key('VALUE'):
                    logging.info('WWW -> GATEWAY : ' + msg['COMMAND'] + ' ' + msg['TOPIC'] + ' ' + msg['VALUE'])
                else:
                    logging.info('WWW -> GATEWAY : ' + msg['COMMAND'] + ' ' + msg['TOPIC'])
                id = str(int(time.time() * 1000000000)) + str(randint(1111, 9999))
                async_response[id] = None
                msg["ID"] = id
                router(self.formatter(msg))
                res = self.poll_response(id)
                if async_response.has_key(id):
                    del async_response[id]
                if res is not None:
                    try:
                        v = str(float(res))
                    except:
                        v = '"'+res+'"'
                    response = '{"ACK":true, "VALUE":' + v + '}'
                else:
                    response = '{"ACK":false}'
                res = 1
                logging.info('GATEWAY -> WWW : ' + str(res))
                try:
                    c.sendall(str(response))
                except:
                    pass
        c.close()

    @staticmethod
    def poll_response(id):
        timeout = time.time() + 3
        while True:
            if async_response.has_key(id):
                if async_response.get(id) is not None:
                    return async_response.pop(id)
                if time.time() > timeout:
                    break
                time.sleep(0.1)
            else: break;
        return None

    @staticmethod
    def formatter(message):
        if message is None:
            return None
        tpc = message["TOPIC"].split("/")
        message["LOCATION"] = tpc[0]
        message["NODE"] = tpc[1]
        message["PARAMETER"] = tpc[2]
        protocol = Db.get_node_protocol(message["LOCATION"],message["NODE"] )
        address = Db.get_node_address(message["LOCATION"], message["NODE"])
        if protocol is None or address is None:
            return None
        else:
            message["PROTOCOL"] = protocol
            message["ADDRESS"] = address
            return message


class Cloud:

    device_id = "OAPI005D1YE7WLSI"
    auth_server = "sensorcloud.microstrain.com"
    my_device = "/SensorCloud/devices/" + device_id
    key = "ed54e079bc69beebf83089457fe7b824b46d6cf5bfe8b5c3870ce50c5f61e638"
    token = ""
    server = ""
    version = 1
    trail = "/?version=" + str(version) + "&auth_token="
    auth_fails = 0

    def __init__(self):
        self.auth()

    def auth(self):
        if self.auth_fails > 9: return False
        try:
            conn = httplib.HTTPSConnection(self.auth_server)
        except:
            return False
        logging.info('GATEWAY -> SENSORCLOUD : AUTH')
        headers = {"Accept": "application/xdr"}
        url = "/SensorCloud/devices/OAPI005D1YE7WLSI/authenticate/?version=1&key=" + self.key
        conn.request('GET', url=url, headers=headers)
        response = conn.getresponse()
        if response.status == httplib.OK:
            data = response.read()
            unpacker = xdrlib.Unpacker(data)
            self.token = unpacker.unpack_string()
            self.server = unpacker.unpack_string()
            self.auth_fails = 0
            logging.info('GATEWAY -> SENSORCLOUD : AUTH OK')
            logging.info('GATEWAY -> SENSORCLOUD : AUTH TOKEN : ' + self.token)
            logging.info('GATEWAY -> SENSORCLOUD : AUTH SERVER : ' + self.server)
            return True
        self.auth_fails += 1
        return False

    def add_sensor(self, location_name, node_name, node_desc=None):
        if node_desc is None: node_desc = location_name + "_" + node_name
        try:
            conn = httplib.HTTPSConnection(self.server)
        except:
            return False
        logging.info('GATEWAY -> SENSORCLOUD : ADD SENSOR : ' + location_name + "_" + node_name)
        url = self.my_device + "/sensors/" + location_name + "_" + node_name + self.trail + self.token
        headers = {"Content-type": "application/xdr"}
        packer = xdrlib.Packer()
        packer.pack_int(self.version)
        packer.pack_string(node_desc)
        packer.pack_string(node_desc)
        packer.pack_string(node_desc)
        data = packer.get_buffer()
        conn.request('PUT', url=url, body=data, headers=headers)
        response = conn.getresponse()
        if response.status == httplib.UNAUTHORIZED:
            if self.auth():
                return self.add_sensor(location_name, node_name)
        elif response.status == httplib.CREATED:
            return True
        return False

    def add_channel(self, location_name, node_name, sensor_name, sensor_desc=None):
        if sensor_desc is None: sensor_desc = sensor_name
        try:
            conn = httplib.HTTPSConnection(self.server)
        except:
            return False
        logging.info('GATEWAY -> SENSORCLOUD : ADD CHANNEL : ' + location_name + "_" + node_name + "/" + sensor_name)
        url = self.my_device + "/sensors/" + location_name + "_" + node_name + "/channels/" + sensor_name + self.trail + self.token
        headers = {"Content-type": "application/xdr"}
        packer = xdrlib.Packer()
        packer.pack_int(self.version)
        packer.pack_string(sensor_desc)
        packer.pack_string(sensor_desc)
        data = packer.get_buffer()
        conn.request('PUT', url=url, body=data, headers=headers)
        response = conn.getresponse()
        if response.status == httplib.CREATED:
            return True
        if response.status == httplib.UNAUTHORIZED:
            if self.auth():
                return self.add_channel(location_name, node_name, sensor_name)
        elif response.status == httplib.NOT_FOUND:
            if self.add_sensor(location_name, node_name):
                return self.add_channel(location_name, node_name, sensor_name)
        return False

    def publish_worker(self, location_name, node_name, sensor_name, value=0):
        try:
            conn = httplib.HTTPSConnection(self.server)
        except:
            return False
        url = self.my_device + "/sensors/" + location_name + "_" + node_name + "/channels/" + sensor_name + "/streams/timeseries/data" + self.trail + self.token
        packer = xdrlib.Packer()
        packer.pack_int(self.version)
        packer.pack_enum(0)  # freq seconds
        packer.pack_int(30)  # num of secs
        packer.pack_int(1)  # points
        timestamp_nanoseconds = int(time.time() * 1000000000)  # start time in nanoseconds
        packer.pack_hyper(timestamp_nanoseconds)
        packer.pack_float(value)
        data = packer.get_buffer()
        headers = {"Content-type": "application/xdr"}
        conn.request('POST', url=url, body=data, headers=headers)
        response = conn.getresponse()
        logging.info('GATEWAY -> SENSORCLOUD : PUBLISH : ' + location_name + "_" + node_name + "/"
                     + sensor_name + " " + str(response.status))
        if response.status == httplib.CREATED:
            return True
        elif response.status == httplib.UNAUTHORIZED:
            if self.auth():
                return self.publish_worker(location_name, node_name, sensor_name, value)
        elif response.status == httplib.NOT_FOUND:
            if self.add_channel(location_name, node_name, sensor_name):
                return self.publish_worker(location_name, node_name, sensor_name, value)
        return False

    def publish (self, location_name, node_name, sensor_name, value=0):
        thread = threading.Thread(target=self.publish_worker, args=(location_name, node_name, sensor_name, value))
        thread.daemon = True
        thread.start()

class Coap:

    def __init__(self):
        logging.info('COAP DAEMON STARTED')
        thread = threading.Thread(target=self.listen,)
        thread.daemon = True
        thread.start()

    @staticmethod
    def formatter(command, topic, value, address):
        pyld = dict()
        pyld["PROTOCOL"] = 'COAP'
        pyld["COMMAND"] = command
        pyld["TOPIC"] = topic.encode('ascii', 'ignore')
        tpc = topic.split("/")
        pyld["LOCATION"] = tpc[0].encode('ascii', 'ignore')
        pyld["NODE"] = tpc[1].encode('ascii', 'ignore')
        pyld["PARAMETER"] = tpc[2].encode('ascii', 'ignore')
        pyld["ADDRESS"] = address
        if type(value) == "unicode":
            pyld["VALUE"] = value.encode('ascii', 'ignore')
        else:
            pyld["VALUE"] = value
        return pyld

    def send(self, message):
        thread = threading.Thread(target=self.send_worker, args=(message,))
        thread.daemon = True
        thread.start()

    def send_worker(self, message):
        host = message["ADDRESS"]
        port = 5683
        path = message["PARAMETER"]
        client = HelperClient(server=(host, port))
        if message.has_key("ID"):
            if message.has_key("VALUE"):
                payload = '{"VALUE":' + message["VALUE"] + ',"ID":' + message["ID"] + '}'
            else:
                payload = '{"ID":' + message["ID"] + '}'
        else:
            if message.has_key("VALUE"):
                payload = '{"VALUE":' + message["VALUE"] + '}'
            else:
                payload = '{}'
        if message["COMMAND"] == "SET":
            message["COMMAND"] = "ACK"
            response = client.put(path, payload)
            msg = json.loads(response.payload)
            message["VALUE"] = msg["VALUE"]
        elif message["COMMAND"] == "GET":
            message["COMMAND"] = "RESPOND"
            response = client.get(path)
            msg = json.loads(response.payload)
            message["VALUE"] = msg["VALUE"]
        elif message["COMMAND"] == "DISCOVER":
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            client = HelperClient(server=(host, port), sock=sock)
            client.get(path, "/.well-known/core", )
            client.stop()
        router(message)
        client.stop()
        logging.debug('GATEWAY -> NODE COAP : SEND' + " " + message["TOPIC"] + " " + payload)
        return True

    def listen(self):
        ip = "192.168.1.10"
        port = 5683
        multicast = False
        coapserver = self.Server(ip, port, multicast)
        try:
            coapserver.listen(10)
        except KeyboardInterrupt:
            coapserver.close()

    class child_res(Resource):
        def __init__(self, name="ChildResource", coap_server=None):
            super(Coap.child_res, self).__init__(name, coap_server, visible=True, observable=True, allow_children=True)
            self.payload = ""

        def render_GET(self, request):
            return self

        def render_POST(self, request):
            res = Coap.basic_res()
            res.location_query = request.uri_query
            res.payload = request.payload
            return res

        def render_DELETE(self, request):
            return True

    class basic_res(Resource):
        def __init__(self, name="BasicResource", coap_server=None):
            super(Coap.basic_res, self).__init__(name, coap_server, visible=True, observable=True, allow_children=True)
            self.payload = "Basic Resource"
            self.resource_type = "rt1"
            self.content_type = "text/plain"
            self.interface_type = "if1"

        def render_GET(self, request):
            return self

        def render_PUT(self, request):
            self.payload = request.payload
            self.edit_resource(request)
            address =  request.source[0]
            message = Coap.formatter('PUBLISH', self.path[1:], self.payload, address)
            router(message)
            return self

        def render_POST(self, request):
            res = self.init_resource(request, Coap.basic_res())
            return res

        def render_DELETE(self, request):
            return True

    class Server(CoAP):
        def __init__(self, host, port, multicast=False):
            CoAP.__init__(self, (host, port), multicast)
            custom_root = Coap.child_res()
            custom_root.path = "/"
            self.root["/"] = custom_root


class Mqtt:

    def __init__(self):
        logging.info('MQTT DAEMON STARTED')
        self.client = mqtt_paho.Client()
        self.client.on_connect = Mqtt.connect
        self.client.on_message = Mqtt.handle
        self.client.loop_start()
        self.client.connect("127.0.0.1", 1883, 60)
        self.client.publish("NODES/DISCOVER", True)

    def send(self, message):
        if message["COMMAND"] == "SET":
            topic = message["TOPIC"] + "/SET"
        elif message["COMMAND"] == "GET":
            topic = message["TOPIC"] + "/GET"
        elif message["COMMAND"] == "DESCRIBE":
            topic = message["TOPIC"] + "/DESCRIBE"
        else:
            topic = message["TOPIC"]
        if message.has_key("ID"):
            if message.has_key("VALUE"):
                payload = '{"VALUE":' + message["VALUE"] + ',"ID":' + message["ID"] + '}'
            else:
                payload = '{"ID":' + message["ID"] + '}'
        else:
            if message.has_key("VALUE"):
                payload = '{"VALUE":' + message["VALUE"] + '}'
            else:
                payload = '{}'
        self.client.publish(topic, payload)
        logging.debug('GATEWAY -> NODE MQTT : PUBLISH' + " " + topic + " " + payload)
        return True

    @staticmethod
    def formatter(message):
        pyld = dict()
        pyld["PROTOCOL"] = 'MQTT'
        pyld["TOPIC"] = message.topic.encode('ascii', 'ignore')
        tpc = message.topic.split("/")
        pyld["LOCATION"] = tpc[0].encode('ascii', 'ignore')
        pyld["NODE"] = tpc[1].encode('ascii', 'ignore')
        pyld["PARAMETER"] = tpc[2].encode('ascii', 'ignore')
        if len(tpc) > 3:
            pyld["COMMAND"] = tpc[3].encode('ascii', 'ignore')
        else:
            pyld["COMMAND"] = "PUBLISH"
        pyld["ADDRESS"] = pyld["TOPIC"]
        try:
            payload = json.loads(message.payload)
            pyld["VALUE"] = payload["VALUE"].encode('ascii', 'ignore')
            if payload.has_key("ID"):
                pyld["ID"] = payload["ID"]
        except:
            pass
        logging.debug('NODE MQTT -> GATEWAY : MESSAGE  ' + str(pyld))
        return pyld

    @staticmethod
    def handle(client, userdata, msg):
        logging.debug('<< NODE MQTT -> GATEWAY : MSG ' + msg.payload)
        try:
            payload = json.loads(msg.payload)
            value = payload['VALUE'].encode('ascii', 'ignore')
        except:
            value = msg.payload
        if msg.topic == "NODES/DESCRIBE":
            client.subscribe(value)
            logging.info('NODE MQTT -> GATEWAY : DESCRIBE : NODES/DESCRIBE ' + value)
        else:
            message = Mqtt.formatter(msg)
            router(message)

    @staticmethod
    def connect(client, userdata, flags, rc):
        client.subscribe("NODES/DESCRIBE")
        client.publish("NODES/DISCOVER", True)


class Lora:

    def __init__(self, serial):
        logging.info('LORA DAEMON STARTED')
        self.serial = serial
        thread = threading.Thread(target=self.listen,)
        thread.daemon = True
        thread.start()

    def send(self, message):
        thread = threading.Thread(target=self.send_worker, args=(message,))
        thread.daemon = True
        thread.start()

    def send_worker(self, message):
        if message.has_key("COMMAND"):
            command = message["COMMAND"].encode('ascii', 'ignore')
        else:
            return False
        if message.has_key("TOPIC"):
            topic = message["TOPIC"].encode('ascii', 'ignore')
        else:
            return False
        if message.has_key("VALUE"):
            value = message["VALUE"].encode('ascii', 'ignore')
        else:
            value = ""
        if message.has_key("ADDRESS"):
            add = message['ADDRESS'].encode('ascii', 'ignore')
            da = unhexlify(add)
        else:
            return False
        if message.has_key("ID"):
            id = message['ID'].encode('ascii', 'ignore')
            dat = '{"COMMAND":"' + command + '","TOPIC":"' + topic + '","VALUE":' + value + ',"ID":"' +id + '"}'
        else:
            dat = '{"COMMAND":"' + command + '","TOPIC":"' + topic + '","VALUE":' + value + '}'
        msg ='|'+add+dat+'|'
        self.serial.write(bytes(msg.encode('ascii', 'ignore')))
        return True

    @staticmethod
    def formatter(message, address):
        pyld = dict()
        pyld["PROTOCOL"] = 'LORA'
        pyld["ADDRESS"] = address
        pyld["COMMAND"] = message["COMMAND"].encode('ascii', 'ignore')
        if message.has_key("TOPIC"):
            pyld["TOPIC"] = message["TOPIC"].encode('ascii', 'ignore')
            tpc = pyld["TOPIC"].split("/")
            pyld["LOCATION"] = tpc[0].encode('ascii', 'ignore')
            pyld["NODE"] = tpc[1].encode('ascii', 'ignore')
            pyld["PARAMETER"] = tpc[2].encode('ascii', 'ignore')
        if message.has_key("VALUE"):
            if type(message["VALUE"]) == "unicode":
                pyld["VALUE"] = message["VALUE"].encode('ascii', 'ignore')
            else:
                pyld["VALUE"] = message["VALUE"]
        if message.has_key("ID"):
            pyld["ID"] = message["ID"]
        return pyld

    def handle(self, frame):
        try:
            data = json.loads(frame)
            node_address = data['NODE'].encode('ascii', 'ignore')
            payload = data['PAYLOAD']
            message = self.formatter(payload, node_address)
            router(message)
        except:
            pass

    def listen(self):
        while True:
            try:
                if self.serial.inWaiting() > 0:
                    frame = self.serial.readline()
                    self.handle(frame)
            except:
                pass

    def close(self):
        self.serial.close()


class Zigbee:

    def __init__(self, serial):
        logging.info('ZIGBEE DAEMON STARTED')
        self.serial = serial
        self.client = ZigBee(serial, callback=self.handle)

    def send(self, message):
        if message.has_key("COMMAND"):
            command = message["COMMAND"].encode('ascii', 'ignore')
        else:
            return False
        if message.has_key("TOPIC"):
            topic = message["TOPIC"].encode('ascii', 'ignore')
        else:
            return False
        if message.has_key("VALUE"):
            value = message["VALUE"].encode('ascii', 'ignore')
        else:
            value = ""
        if message.has_key("ADDRESS"):
            address = message['ADDRESS'].encode('ascii', 'ignore')
            da = unhexlify(address)
        else:
            return False
        if message.has_key("ID"):
            id = message['ID'].encode('ascii', 'ignore')
            dat = '{"COMMAND":"' + command + '","TOPIC":"' + topic + '","VALUE":' + value + ',"ID":"' + id + '"}'
        else:
            dat = '{"COMMAND":"' + command + '","TOPIC":"' + topic + '","VALUE":' + value + '}'
        self.client.send('tx', frame_id=b'\x01', dest_addr_long=da, broadcast_radius=b'\x00', options=b'\x00', data=dat)
        return True

    @staticmethod
    def formatter(message, address):
        pyld = dict()
        pyld["PROTOCOL"] = 'ZIGBEE'
        pyld["ADDRESS"] = address.encode('ascii', 'ignore')
        pyld["COMMAND"] = message["COMMAND"].encode('ascii', 'ignore')
        if message.has_key("TOPIC"):
            pyld["TOPIC"] = message["TOPIC"].encode('ascii', 'ignore')
            tpc = pyld["TOPIC"].split("/")
            pyld["LOCATION"] = tpc[0].encode('ascii', 'ignore')
            pyld["NODE"] = tpc[1].encode('ascii', 'ignore')
            pyld["PARAMETER"] = tpc[2].encode('ascii', 'ignore')
        if message.has_key("VALUE"):
            if type(message["VALUE"]) == "unicode":
                pyld["VALUE"] = message["VALUE"].encode('ascii', 'ignore')
            else:
                pyld["VALUE"] = message["VALUE"]
        if message.has_key("ID"):
            pyld["ID"] = message["ID"]
        return pyld

    def handle(self, frame):
        id = frame['id']
        if id == 'rx':
            source_addr_long = frame['source_addr_long']
            node_address = hexlify(source_addr_long)
            data = frame['rf_data']
            payload = json.loads(data)
            message = self.formatter(payload, node_address)
            router(message)
        elif id == "tx_status": pass
            #hexlify(frame['deliver_status'])

    def close(self):
        self.client.halt()
        self.serial.close()

logging.basicConfig(filename='iotgateway.log', level=logging.DEBUG)
gateway_logger = logging.getLogger()
gateway_logger.setLevel(level=logging.INFO)
coapthon_logger = logging.getLogger('CoAPthon')
coapthon_logger.setLevel(level=logging.ERROR)

logging.info('GATEWAY INIT STARTED')
rest = Rest()
cloud = Cloud()
coap = Coap()
mqtt = Mqtt()
lora = Lora(serial.Serial('/dev/ttyACM0', 9600))
zigbee = Zigbee(serial.Serial('/dev/serial0', 9600))
async_response = {}
logging.info('GATEWAY INIT FINISHED')
logging.info('#####################')
s_counter = 0
counter = 0

def router(message):
    if message is not None:
        val = ""
        if message.has_key("VALUE"): val = " " + str(message["VALUE"])
        if message["COMMAND"] == "PUBLISH":
            logging.info('NODE ' + message["PROTOCOL"] + " -> GATEWAY : " + message["COMMAND"] + " : " + message["TOPIC"]+val)
            Db.save_sensor_data(message)
            cloud.publish(message["LOCATION"], message["NODE"], message["PARAMETER"], float(message["VALUE"]))
        elif message["COMMAND"] == "DESCRIBE":
            logging.info('GATEWAY -> NODE ' + message["PROTOCOL"] + " : " + message["COMMAND"] + " : " + message["TOPIC"]+val)
        elif message["COMMAND"] == "DISCOVER":
            logging.info('NODE ' + message["PROTOCOL"] + " -> GATEWAY : " + message["COMMAND"] + " : " + message["TOPIC"]+val)
        elif message["COMMAND"] == "ACK":
            id = message["ID"]
            async_response[id] = True
            logging.info('NODE ' + message["PROTOCOL"] + " -> GATEWAY : " + message["COMMAND"] + " : " + message["TOPIC"]+val)
        elif message["COMMAND"] == "RESPOND":
            id = message["ID"]
            async_response[id] = message["VALUE"]
            logging.info('NODE ' + message["PROTOCOL"] + " -> GATEWAY : " + message["COMMAND"] + " : " + message["TOPIC"]+val)
        elif message["COMMAND"] == "SET" or message["COMMAND"] == "GET":
            logging.info('GATEWAY -> NODE ' + message["PROTOCOL"] + " : " + message["COMMAND"] + " : " + message["TOPIC"]+val)
            if message["PROTOCOL"] == "ZIGBEE": return zigbee.send(message)
            elif message["PROTOCOL"] == "LORA": return lora.send(message)
            elif message["PROTOCOL"] == "MQTT": return mqtt.send(message)
            elif message["PROTOCOL"] == "COAP": return coap.send(message)

def main():
    while True:
        try:
            time.sleep(1)
        except KeyboardInterrupt:
            break
    zigbee.close()

if __name__ == '__main__':
    main()
