
CREATE TABLE `data` (
  `id` int(11) NOT NULL,
  `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `sensor_id` int(10) UNSIGNED DEFAULT NULL,
  `sensor_data` decimal(10,2) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4; 

CREATE TABLE `data_full` (
`data_id` int(11)
,`data_date` timestamp
,`data` decimal(10,2)
,`topic` varchar(194)
,`location_name` varchar(64)
,`node_name` varchar(64)
,`sensor_name` varchar(64)
,`node_protocol` varchar(16)
,`node_address` varchar(64)
,`sensor_id` int(10) unsigned
,`node_id` int(10) unsigned
,`location_id` int(10) unsigned
);

CREATE TABLE `location` (
  `id` int(10) UNSIGNED NOT NULL,
  `name` varchar(64) NOT NULL,
  `description` varchar(128) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE `node` (
  `id` int(10) UNSIGNED NOT NULL,
  `location_id` int(10) UNSIGNED NOT NULL,
  `name` varchar(64) NOT NULL,
  `description` varchar(128) DEFAULT NULL,
  `date_added` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `protocol` varchar(16) NOT NULL,
  `address` varchar(64) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE `protocol` (
  `id` int(10) UNSIGNED NOT NULL,
  `name` varchar(16) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE `sensor` (
  `id` int(10) UNSIGNED NOT NULL,
  `node_id` int(10) UNSIGNED NOT NULL,
  `name` varchar(64) NOT NULL,
  `description` varchar(128) DEFAULT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE `sensor_full` (
`topic` varchar(194)
,`location_name` varchar(64)
,`node_name` varchar(64)
,`sensor_name` varchar(64)
,`node_protocol` varchar(16)
,`node_address` varchar(64)
,`location_id` int(10) unsigned
,`node_id` int(10) unsigned
,`sensor_id` int(10) unsigned
);
DROP TABLE IF EXISTS `data_full`;

CREATE ALGORITHM=UNDEFINED DEFINER=`root`@`localhost` SQL SECURITY DEFINER VIEW `data_full`  AS  select `data`.`id` AS `data_id`,`data`.`date` AS `data_date`,`data`.`sensor_data` AS `data`,concat(`location`.`name`,'/',`node`.`name`,'/',`sensor`.`name`) AS `topic`,`location`.`name` AS `location_name`,`node`.`name` AS `node_name`,`sensor`.`name` AS `sensor_name`,`node`.`protocol` AS `node_protocol`,`node`.`address` AS `node_address`,`data`.`sensor_id` AS `sensor_id`,`node`.`id` AS `node_id`,`location`.`id` AS `location_id` from (((`data` join `sensor`) join `node`) join `location`) where ((`data`.`sensor_id` = `sensor`.`id`) and (`sensor`.`node_id` = `node`.`id`) and (`node`.`location_id` = `location`.`id`)) ;
DROP TABLE IF EXISTS `sensor_full`;

CREATE ALGORITHM=UNDEFINED DEFINER=`root`@`localhost` SQL SECURITY DEFINER VIEW `sensor_full`  AS  select concat(`location`.`name`,'/',`node`.`name`,'/',`sensor`.`name`) AS `topic`,`location`.`name` AS `location_name`,`node`.`name` AS `node_name`,`sensor`.`name` AS `sensor_name`,`node`.`protocol` AS `node_protocol`,`node`.`address` AS `node_address`,`location`.`id` AS `location_id`,`node`.`id` AS `node_id`,`sensor`.`id` AS `sensor_id` from ((`sensor` left join `node` on((`sensor`.`node_id` = `node`.`id`))) left join `location` on((`node`.`location_id` = `location`.`id`))) ;

ALTER TABLE `data`
  ADD PRIMARY KEY (`id`),
  ADD KEY `data_sensor` (`sensor_id`);

ALTER TABLE `location`
  ADD PRIMARY KEY (`id`),
  ADD UNIQUE KEY `name` (`name`);

ALTER TABLE `node`
  ADD PRIMARY KEY (`id`),
  ADD UNIQUE KEY `node_location_id` (`location_id`,`id`) USING BTREE,
  ADD KEY `protocol` (`protocol`);

ALTER TABLE `protocol`
  ADD PRIMARY KEY (`id`),
  ADD UNIQUE KEY `name` (`name`);

ALTER TABLE `sensor`
  ADD PRIMARY KEY (`id`),
  ADD UNIQUE KEY `node_sensor` (`id`,`node_id`) USING BTREE,
  ADD KEY `node_id` (`node_id`);

ALTER TABLE `data`
  ADD CONSTRAINT `data_sensor` FOREIGN KEY (`sensor_id`) REFERENCES `sensor` (`id`);

ALTER TABLE `node`
  ADD CONSTRAINT `node_location_id` FOREIGN KEY (`location_id`) REFERENCES `location` (`id`),
  ADD CONSTRAINT `node_protocol` FOREIGN KEY (`protocol`) REFERENCES `protocol` (`name`);

ALTER TABLE `sensor`
  ADD CONSTRAINT `sensor_node` FOREIGN KEY (`node_id`) REFERENCES `node` (`id`);


