#include "Arduino.h"

#define MQTT "MQTT"
#define COAP "COAP"
#define XBEE "XBEE"
#define LORA "LORA"

#define DISCOVER "DISCOVER"
#define DESCRIBE "DESCRIBE"
#define PUBLISH  "PUBLISH"
#define SET "SET"
#define GET "GET"
#define ACK "ACK"
#define RESPOND "RESPOND"

struct Message {
	char command[16];
	char topic[128];
	char value[128];
	char id[32];
};

struct Topic {
    char location[40];
    char node[40];
    char parameter[40];
    char trail[16];
};

class Node {
	
  public:
  
	void set_location(char*);
	void set_name(char*);
  
  	void get_parameter_callback(char*(*get_parameter)(char*));
	void set_parameter_callback(bool (*set_parameter)(char*, char*));
	void describe_callback(void (*describe)(void));
	
	static char* to_char_buff( char*);
    static char* to_char_buff( bool);
	static char* to_char_buff( int);
	static char* to_char_buff( long);
	static char* to_char_buff( unsigned long);
	static char* to_char_buff( long long);
	static char* to_char_buff( double);
	static char* to_char_buff( float);
	
  protected:
	
	Node();
  
	char* _name = "UNKNOWN";
	char* _location = "UNKNOWN";;
	char* _protocol = "UNKNOWN";;
	
	char* (*get_parameter)(char*);
	bool  (*set_parameter)(char*, char*);
	void  (*describe)(void);

    static Topic split_topic(char*);
    static Message split_message(char*);
	static char* get_parameter_from_topic(char*); 
	char* generate_topic( char*);
	char* generate_topic( char*, char*);
		
};

