#include "Arduino.h"
#include "NodeCOAP.h"


NodeCOAP::NodeCOAP(){}

NodeCOAP::NodeCOAP(Coap& client){
	set_client(client);
}

void NodeCOAP::set_client(Coap& client){
	this->_client = &client;
}

void NodeCOAP::subscribe_callback(void (*sus_callback)(CoapPacket&, IPAddress, int, char*)){
	this->sus_callback = sus_callback;
}

void NodeCOAP::response_callback(void (*res_callback)(CoapPacket&, IPAddress, int, char*)){
	this->_client->response(res_callback);
}

void NodeCOAP::init( IPAddress server, uint16_t port ){
	this->_gateway_address = server;
	this->_port = port;
	this->_client->start();
	describe();
	this->_client->server(this->sus_callback, ".well-known/core");
}

void NodeCOAP::loop(){
	this->_client->loop();
}

void NodeCOAP::subscribe(char *topic){
	char tpc[128]="";
	sprintf(tpc, "%s", topic);
	int msgid = _client->get(this->_gateway_address, this->_port, tpc);
}

void NodeCOAP::publish(char *parameter, char *value){
	char topic[128]="";
	sprintf(topic, "%s/%s/%s",this->_location, this->_name, parameter);
	int msgid = _client->put(this->_gateway_address, this->_port, topic, value);
}
	
void NodeCOAP::publish(char *parameter, bool value){
	char *v = value ? "1" : "0";
	publish(parameter, v);
}

void NodeCOAP::publish(char *parameter, int value){
	char v[8]="";
	sprintf(v,"%d", value);
	publish(parameter, v);
}

void NodeCOAP::publish(char *parameter, long value){
	char v[16]="";
	sprintf(v,"%ld", value);
	publish(parameter, v);
}

void NodeCOAP::publish(char *parameter, double value){
	char v[16]="";
	dtostrf (value, 0, 4, v);
	publish(parameter, v);
}

void NodeCOAP::publish(char *parameter, float value){
	char v[16]="";
	dtostrf (value, 0, 4, v);
	publish(parameter, v);
}

void NodeCOAP::subscribe_handle(CoapPacket &packet, IPAddress ip, int port,  char* parameter){
	char p[packet.payloadlen + 1];
	memcpy(p, packet.payload, packet.payloadlen);
	p[packet.payloadlen] = NULL;
	Message msg = split_message((char *) p);
	msg.value;
	msg.id;
	char payload[256]="";
	if (packet.code == 1){
		if(!strcmp(parameter, ".well-known/core")){
			this->_client->sendResponse(ip, port, packet.messageid, this->well_known, strlen(this->well_known),
										COAP_CONTENT, COAP_APPLICATION_LINK_FORMAT, NULL, 0);
		}else{
			char* value = get_parameter(parameter);
			char v[strlen(value) + 1];
			memcpy(v, value, strlen(value));
			v[strlen(value)] = NULL;
			sprintf(payload, "{\"VALUE\":\"%s\",\"ID\":\"%s\"}", v, msg.id);
			this->_client->sendResponse(ip, port, packet.messageid, payload);
		}
	}else if (packet.code == 2 || packet.code == 3){
		set_parameter(parameter, msg.value);
		sprintf(payload, "{\"VALUE\":\"%s\",\"ID\":\"%s\"}", msg.value, msg.id);
		this->_client->sendResponse(ip, port, packet.messageid, payload);

	}
}

void NodeCOAP::describe_parameter(char *parameter, bool settable = false){
	this->_client->server(this->sus_callback, parameter);
	strcat(this->well_known,"</");
	strcat(this->well_known,parameter);
	strcat(this->well_known,">;rt=\"rt1\";obs;if=\"if1\",");
	desc(parameter);
}

void NodeCOAP::desc( char *parameter){
	char topic[128]="";
	sprintf(topic, "%s/%s/%s",this->_location, this->_name, parameter);
	int msgid = _client->post(this->_gateway_address, this->_port, topic);
}

static void NodeCOAP::callback_resp(CoapPacket &packet, IPAddress ip, int port, char* url) {
	if (packet.code == COAP_NOT_FOUNT){
	}
	char p[packet.payloadlen + 1];
	memcpy(p, packet.payload, packet.payloadlen);
	p[packet.payloadlen] = NULL;
}
