#include "Arduino.h"
#include "NodeMQTT.h"

char* NodeMQTT::generate_payload(char *value, char *id = NULL){
	char payload[256]="";
	if (id == NULL){
		sprintf(payload, "{\"VALUE\":\"%s\"}", value);
	}else{
		sprintf(payload, "{\"VALUE\":\"%s\",\"ID\":\"%s\"}", value, id);
	}
	return payload;
}

bool NodeMQTT::reconnect(){
	while (!_client->connected()) {		
		String clientId = "ESP8266Client-";
		clientId += String(random(0xffff), HEX);
		if (_client->connect(clientId.c_str())) {
		  describe();
		  this->_client->subscribe("NODES/DISCOVER");
		} else {
		  delay(5000);
		}
	}
}

NodeMQTT::NodeMQTT(){}

NodeMQTT::NodeMQTT(PubSubClient& client){
	set_client(client);
}

void NodeMQTT::set_client(PubSubClient& client){
	this->_client = &client;
}

void NodeMQTT::subscribe_callback(void (*callback)(char*, uint8_t*, unsigned int)){
	this->_client->setCallback(callback);
}

bool NodeMQTT::subscribe_handle(char* topic, uint8_t* payload, unsigned int length) {
	if (!strcmp(topic, "NODES/DISCOVER")) {
		describe();
	}else{
		Topic tpc = split_topic(topic);
		Message msg = split_message((char *) payload);
		if (!strcmp(tpc.trail, "GET")) {
			sprintf(msg.value, "%s", get_parameter(tpc.parameter));
			Serial.println(msg.value);
			publish(tpc.parameter, msg.value, "RESPOND", msg.id);
		}else if (!strcmp(tpc.trail, "SET")) {
			bool result = set_parameter(tpc.parameter, msg.value);
			publish(tpc.parameter, result ,"ACK", msg.id);
		}
	}
	return true;
}

void NodeMQTT::init( IPAddress server, uint16_t port ){
	this->_client->setServer(server, port);
    this->lastReconnectAttempt = 0;
}

void NodeMQTT::loop(){
	if (!_client->connected()) {
		reconnect();
	}
	_client->loop();
	
}

void NodeMQTT::subscribe(char *topic, char *action = NULL){
	bool result;
	if (action == NULL){
		result = _client->subscribe(topic);	
	}else{
		char tpc[128]="";
		sprintf(tpc, "%s/%s",topic,  action);
		result = _client->subscribe(tpc);
	}
}

void NodeMQTT::publish(char *parameter, char *value, char *action = NULL, char *id = NULL){
	char topic[128]="";
	if (action == NULL){
		sprintf(topic, "%s/%s/%s",this->_location, this->_name, parameter);
	}else{
		sprintf(topic, "%s/%s/%s/%s",this->_location, this->_name, parameter, action);
	}
	if (id == NULL){
		this->_client->publish(topic, generate_payload(value));
	}else{
		this->_client->publish(topic, generate_payload(value,id));
	}
	Serial.println(topic);
	Serial.println(generate_payload(value,id));
}

void NodeMQTT::publish(char *parameter, bool value, char *action = NULL, char *id = NULL){
	char *v = value ? "1" : "0";
	publish(parameter, v, action, id);
}

void NodeMQTT::publish(char *parameter, int value, char *action = NULL, char *id = NULL){
	char v[8]="";
	sprintf(v,"%d", value);
	publish(parameter, v, action, id);
}

void NodeMQTT::publish(char *parameter, long value, char *action = NULL, char *id = NULL){
	char v[16]="";
	sprintf(v,"%ld", value);
	publish(parameter, v, action, id);
}

void NodeMQTT::publish(char *parameter, double value, char *action = NULL, char *id = NULL){
	char v[16]="";
	dtostrf (value, 0, 4, v);
	publish(parameter, v, action, id);
}

void NodeMQTT::publish(char *parameter, float value, char *action = NULL, char *id = NULL){
	char v[16]="";
	dtostrf (value, 0, 4, v);
	publish(parameter, v, action, id);
}

void NodeMQTT::describe_parameter(char *parameter, bool settable = false){
	_client->publish("NODES/DESCRIBE", generate_payload(generate_topic(parameter))); 
	_client->publish("NODES/DESCRIBE", generate_topic(parameter, "RESPOND"));
	subscribe(generate_topic(parameter, "GET"));
	if (settable){
		_client->publish("NODES/DESCRIBE", generate_topic(parameter, "ACK"));
		subscribe(generate_topic(parameter, "SET"));
	}
}
