#include "Arduino.h"
#include "NodeUART.h"

void NodeUART::publish(char *parameter, char *value){
	char v[128]="";
	sprintf(v,"\"%s\"", value);
	tx(encode(PUBLISH, parameter, v));
}

void NodeUART::publish(char *parameter, bool value){
	char *v = value ? "1" : "0";
	tx(encode(PUBLISH, parameter, v));
}

void NodeUART::publish( char *parameter, int value){
	char v[8]="";
	sprintf(v,"%d", value);
	tx(encode(PUBLISH, parameter, v));
}

void NodeUART::publish( char *parameter, long value){
	char v[16]="";
	sprintf(v,"%ld", value);
	tx(encode(PUBLISH, parameter, v));
}

void NodeUART::publish( char *parameter, double value){
	char v[16]="";
	dtostrf (value, 0, 4, v);
	tx(encode(PUBLISH, parameter, v));
}

void NodeUART::publish( char *parameter, float value){
	Serial.println(value);
	char v[16]="";
	dtostrf (value, 0, 4, v);
	tx(encode(PUBLISH, parameter, v));
}

NodeUART::NodeUART(){}

char* NodeUART::encode( char *command, char *parameter = NULL, char *value = NULL, char* id = NULL){
	char payload[256]="";
	if (parameter){
		if(value){
			if(id){
				sprintf(payload, "{\"COMMAND\":\"%s\",\"TOPIC\":\"%s\",\"VALUE\":%s,\"ID\":\"%s\"}", command,
						this->generate_topic(parameter), value, id);
			}else{
				sprintf(payload, "{\"COMMAND\":\"%s\",\"TOPIC\":\"%s\",\"VALUE\":%s}", command, this->generate_topic(parameter),value);
			}
		}else{
			if(id){
				sprintf(payload, "{\"COMMAND\":\"%s\",\"TOPIC\":%s,\"ID\":\"%s\"}", command, this->generate_topic(parameter),id);
			}else {
				sprintf(payload, "{\"COMMAND\":\"%s\",\"TOPIC\":%s}", command, this->generate_topic(parameter));
			}
		}
	}else{
		if(id){
			sprintf(payload, "{\"COMMAND\":\"%s\",\"ID\":\"%s\"}", command, id);
		}else {
			sprintf(payload, "{\"COMMAND\":\"s\"}", command);
		}
	}
	return payload;
}

void  NodeUART::decode(char *payload){
	Message msg;
	char *end_payload;
	char *pair = strtok_r(payload, "}{,", &end_payload);
	while (pair != NULL) {
		char *end_pair;
		char key[16]="";
		char val[128]="";
		sscanf(pair," \"%[^\"]\" : %s ", key, val);
		char *value = strtok_r(val, "\"", &end_pair);
		if (!strcmp(key, "COMMAND")){
		  strncpy(msg.command, value, 16);
		}else if(!strcmp(key, "TOPIC")){
		  strncpy(msg.topic, value, 128);
		}else if(!strcmp(key, "VALUE")){
		  strncpy(msg.value, value, 128);
		}else if(!strcmp(key, "ID")){
		  strncpy(msg.id, value, 32);
		}
		pair = strtok_r(NULL, "}{,", &end_payload);
	}
  char* parameter = get_parameter_from_topic(msg.topic);
  if (!strcmp(msg.command, "DES")){
    //this->des();
  }else if(!strcmp(msg.command, "SET")){
  	set_parameter(parameter, msg.value);
    ack(parameter, true, msg.id);
  }else if(!strcmp(msg.command, "GET")){
  	char* val = get_parameter(parameter);
  	if (val != NULL) respond( parameter, val, msg.id);
  }
}

char* NodeUART::des( char *parameter, bool value = false){
	char *v = value ? "1" : "0";
	return this->encode(DESCRIBE, parameter, v);
}

char* NodeUART::ack( char *parameter, bool result, char *id){
	char *v = result ? "1" : "0";
	tx(encode(ACK, parameter, v, id));
}

void NodeUART::respond(char *parameter, char *value, char *id){
	char v[128]="";
	sprintf(v,"\"%s\"", value);
	tx(encode(RESPOND, parameter, v, id));
}

void NodeUART::describe_parameter(char *parameter, bool settable = false){
}

void NodeUART::describe(){
}


