#include <DHT.h>
#include <coap.h>
#include <WiFiEsp.h>
#include <WiFiEspUdp.h>
#include <NodeCOAP.h>

#define L_PIN 8
#define DHTPIN 7     
#define DHTTYPE DHT11  

DHT dht(DHTPIN, DHTTYPE);
  
#define LOCATION "OFFICES"
#define NODE "ONE" 

#define TEMPERATURE "TEMPERATURE" 
#define HUMIDITY "HUMIDITY" 
#define LIGHT "LIGHT" 

long unsigned lastSendTime = 0;           
long unsigned interval = 60000;          

char ssid[] = "XXXXXXXXXXXXXX";                          
char pass[] = "XXXXXXXXXXXXXX";     
int status = WL_IDLE_STATUS;                      
IPAddress server(192, 168, 1, 10);

WiFiEspUDP Udp;
Coap coap(Udp);
NodeCOAP node(coap);


char* get_parameter_callback(char* parameter){ 
  if(!strcmp(parameter, LIGHT)){
    return Node::to_char_buff(digitalRead(L_PIN));
  }else if (!strcmp(parameter, TEMPERATURE)){
    float temperature = dht.readTemperature();
    if  (!isnan(temperature))  return Node::to_char_buff(dht.readTemperature());
  }else if(!strcmp(parameter, HUMIDITY)){
    float humidity = dht.readHumidity();
    if  (!isnan(humidity))  return Node::to_char_buff(dht.readHumidity());
  }
  return NULL;
}

bool set_parameter_callback(char* parameter, char *value){ 
    if(!strcmp(parameter, LIGHT)){
        bool v = value[0] == '1' ? HIGH : LOW;
        digitalWrite(L_PIN, v); 
        return true;
    }
    return false;
} 

void describe_callback(){ ;
   node.describe_parameter(TEMPERATURE);
   node.describe_parameter(HUMIDITY);
   node.describe_parameter(LIGHT, 1); 
}

void subscribe_callback(CoapPacket &packet, IPAddress ip, int port, char* parameter) {
  node.subscribe_handle(packet,ip, port, parameter);
}

void response_callback(CoapPacket &packet, IPAddress ip, int port, char* parameter) {
  if (packet.code == COAP_NOT_FOUNT){
    node.desc(TEMPERATURE);
    node.desc(HUMIDITY);
    node.desc(LIGHT); 
  }
}

void setup_node(){
  node.response_callback(response_callback);
  node.set_location(LOCATION);
  node.set_name(NODE);
  node.get_parameter_callback(get_parameter_callback);
  node.set_parameter_callback(set_parameter_callback);
  node.subscribe_callback(subscribe_callback);
  node.describe_callback(describe_callback);
  node.init(server, 5683);
  delay(1000);
}

void setup_wifi(){
  WiFi.init(&Serial);
  if (WiFi.status() == WL_NO_SHIELD) {
    while (true);
  }
  WiFi.begin(ssid, pass);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
  randomSeed(micros());
}


void setup() {
  pinMode(9, OUTPUT);
  digitalWrite(9, LOW);
  Serial.begin(115200);
  dht.begin();
  setup_wifi();
  setup_node();
  delay(1000);
}

void loop()
{
  node.loop();
  if (millis() - lastSendTime > interval) {
    float humidity = dht.readHumidity();
    float temperature = dht.readTemperature();
    if (isnan(humidity) || isnan(temperature)) {
    }else{
      node.publish(TEMPERATURE, temperature);
      node.publish(HUMIDITY, humidity);
    }
    lastSendTime = millis();
  }
}
