#include <DHT.h>
#include <WiFiEsp.h>
#include <PubSubClient.h>
#include <NodeMQTT.h>

#define L_PIN 8
#define DHTPIN 7     
#define DHTTYPE DHT11  

DHT dht(DHTPIN, DHTTYPE);
  
#define LOCATION "LABORATORY"
#define NODE "ONE" 

#define TEMPERATURE "TEMPERATURE" 
#define HUMIDITY "HUMIDITY" 
#define LIGHT "LIGHT" 

long unsigned lastSendTime = 0;           
long unsigned interval = 60000;          

char ssid[] = "XXXXXXXX";                          
char pass[] = "XXXXXXXXXXXX";        
int status = WL_IDLE_STATUS;                      
IPAddress server(192, 168, 1, 10);

WiFiEspClient wifiClient;
PubSubClient mqtt(wifiClient);
NodeMQTT node(mqtt);


char* get_parameter_callback(char* parameter){ 
	if(!strcmp(parameter, LIGHT)){
		return Node::to_char_buff(digitalRead(L_PIN));
	}else if (!strcmp(parameter, TEMPERATURE)){
		return Node::to_char_buff(dht.readTemperature());
	}else if(!strcmp(parameter, HUMIDITY)){
		return Node::to_char_buff(dht.readHumidity());
	}
}

bool set_parameter_callback(char* parameter, char *value){ 
    if(!strcmp(parameter, LIGHT)){
        bool v = value[0] == '1' ? HIGH : LOW;
        digitalWrite(L_PIN, v); 
        return true;
    }
    return false;
}

void describe_callback(){ 
   node.describe_parameter(TEMPERATURE);
   node.describe_parameter(HUMIDITY);
   node.describe_parameter(LIGHT, 1); 
}

bool subscribe_callback(char* topic, uint8_t* payload, unsigned int length){ 
    node.subscribe_handle(topic,payload,length);
}

void setup_node(){
  node.init(server, 1883);
  node.set_location(LOCATION);
  node.set_name(NODE);
  node.get_parameter_callback(get_parameter_callback);
  node.set_parameter_callback(set_parameter_callback);
  node.subscribe_callback(subscribe_callback);
  node.describe_callback(describe_callback);
}

void setup_wifi(){
  WiFi.init(&Serial);
  if (WiFi.status() == WL_NO_SHIELD) {
    while (true);
  }
  WiFi.begin(ssid, pass);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
  randomSeed(micros());
}

void setup(){
  Serial.begin(115200);
  dht.begin();
  setup_wifi();
  setup_node();
}

void loop(){
  node.loop();
  if (millis() - lastSendTime > interval) {
    float humidity = dht.readHumidity();
    float temperature = dht.readTemperature();
    if (isnan(humidity) || isnan(temperature)) {
    }else{;
      node.publish(TEMPERATURE, temperature);
      node.publish(HUMIDITY, humidity);
    }
    lastSendTime = millis();
  }
}
