#include <NodeZIGBEE.h>
#include <XBee.h>
#include <DHT.h>

#define L_PIN 8
#define DHTPIN 7     
#define DHTTYPE DHT11  

#define LOCATION "GREENHOUSE"
#define NODE "ONE" 

#define TEMPERATURE "TEMPERATURE" 
#define HUMIDITY "HUMIDITY" 
#define LIGHT "LIGHT" 

long lastSendTime = 0;        
long interval = 60000;   

DHT dht(DHTPIN, DHTTYPE);

XBee xbee = XBee();
ZBRxResponse rx = ZBRxResponse();
ModemStatusResponse msr = ModemStatusResponse();
NodeZIGBEE node(xbee, rx, msr);


char* get_parameter_callback(char* parameter){ 
  if(!strcmp(parameter, LIGHT)){
    return Node::to_char_buff(digitalRead(L_PIN));
  }else if (!strcmp(parameter, TEMPERATURE)){
    float temperature = dht.readTemperature();
    if  (!isnan(temperature))  return Node::to_char_buff(dht.readTemperature());
  }else if(!strcmp(parameter, HUMIDITY)){
    float humidity = dht.readHumidity();
    if  (!isnan(humidity))  return Node::to_char_buff(dht.readHumidity());
  }
  return NULL;
}

void set_parameter_callback(char* parameter, char *value){ 
    if(!strcmp(parameter, LIGHT)){
        bool v = value[0] == '1' ? HIGH : LOW;
        digitalWrite(L_PIN, v); 
    }
}

void describe_callback() { 
   node.describe_parameter(TEMPERATURE);
   node.describe_parameter(HUMIDITY);
   node.describe_parameter(LIGHT, 1); 
}

void setup() {  
  pinMode(L_PIN, OUTPUT); 
  digitalWrite(L_PIN, LOW); 
  dht.begin();
  Serial.begin(9600);
  node.init(Serial);
  node.set_location(LOCATION);
  node.set_name(NODE);
  node.describe_callback(describe_callback);
  node.get_parameter_callback(get_parameter_callback);
  node.set_parameter_callback(set_parameter_callback);
  delay(1000);
}

void loop() {
  if (millis() - lastSendTime > interval) {
    float humidity = dht.readHumidity();
    float temperature = dht.readTemperature();
    if (isnan(humidity) || isnan(temperature)) {
    }else{
      node.publish(TEMPERATURE, temperature);
      node.publish(HUMIDITY, humidity);
    }
    lastSendTime = millis();
  }
  node.loop();
}

